@@ -3,6 +3,7 @@ use ethnum::U256;
 use tap::Pipe;
 
 use crate::{
+    typesys::{Type, Variable},
     containers::{List, Map, Set, Symbol},
     context::{Ctx, ModuleId},
 };
@@ -68,7 +69,8 @@ pub enum RawConstExpr {
 /// A raw expression.
 #[derive(Clone, Debug)]
 pub enum RawExpr {
-    Let(Ctx<Symbol>, Ctx<Self>, Ctx<Self>),
+    //Let(Ctx<Symbol>, Ctx<Self>, Ctx<Self>),
+    Let(List<(Ctx<Symbol>, Ctx<Self>)>, Ctx<Self>),
     For(Ctx<Symbol>, Ctx<Self>, Ctx<Self>),
     ForFold(Ctx<Symbol>, Ctx<Self>, Ctx<Symbol>, Ctx<Self>, Ctx<Self>),
     If(Ctx<Self>, Ctx<Self>, Ctx<Self>),
@@ -172,7 +174,7 @@ fn visit(
                 log::trace!("{:?} depends on {:?}", def.name(), parent);
                 visit(parent, defs, &mut sorted, &mut visited);
             }
-            sorted.push_back(def.clone());
+            sorted.push(def.clone());
         }
     }
 }
@@ -286,13 +288,29 @@ fn typebind_parents(tb: &RawTypeExpr) -> Set<Symbol> {
     }
 }
 
+impl RawExpr {
+    pub fn free_variables<Tv: Variable, Cv: Variable>(&self, var_set: &Map<Symbol, Type<Tv,Cv>>)
+    -> Set<Symbol> {
+        let vars = expr_parents(self);
+        vars.into_iter().filter(|sym| var_set.contains_key(sym)).collect()
+    }
+}
+
 fn expr_parents(expr: &RawExpr) -> Set<Symbol> {
     let rec = expr_parents;
 
     match expr {
         // Important cases
         RawExpr::Var(var) => Set::unit(*var),
-        RawExpr::Let(var, val, body) => rec(val).union(rec(body)).without(var),
+        //RawExpr::Let(var, val, body) => rec(val).union(rec(body)).without(var),
+        RawExpr::Let(binds, body) => {
+            let vars: Vec<Ctx<Symbol>> = binds.iter().map(|(v,_)| v.clone()).collect();
+            let vals: Vec<Ctx<RawExpr>> = binds.iter().map(|(_,v)| v.clone()).collect();
+            vars.iter().fold(
+                vals.iter().fold(Set::new(), |acc, v| acc.union(rec(v)))
+                    .union(rec(body)),
+                |acc,var| acc.without(var))
+        }
         RawExpr::Apply(fn_name, args) => args
             .into_iter()
             .fold(rec(fn_name), |acc, arg| acc.union(rec(arg))),
